
CREATE TABLE Objects 
  ( 
     id          VARCHAR PRIMARY KEY NOT NULL,
     objectClass VARCHAR
  );;
  
CREATE TABLE Events 
  ( 
     id              VARCHAR PRIMARY KEY NOT NULL, 
     identifier      VARCHAR, 
     name            VARCHAR, 
     info            VARCHAR, 
     location        VARCHAR,
     logoId          VARCHAR REFERENCES Attachments(id) DEFERRABLE INITIALLY DEFERRED,
     startDate       DATE, 
     endDate         DATE, 
     defaultTimeZone VARCHAR, 
     release         INTEGER
  );;

CREATE TRIGGER Events_Object_Delete AFTER DELETE ON Events
BEGIN
    DELETE FROM Objects WHERE Objects.id = OLD.id;
END;;

CREATE TABLE Types 
  ( 
     id        VARCHAR PRIMARY KEY NOT NULL, 
     eventId   VARCHAR NOT NULL, 
     naturalId VARCHAR, 
     name      VARCHAR,
     FOREIGN KEY (eventId) REFERENCES Events(id) ON DELETE CASCADE DEFERRABLE INITIALLY DEFERRED
  );;

CREATE TRIGGER Types_Object_Delete AFTER DELETE ON Types 
BEGIN
    DELETE FROM Objects WHERE Objects.id = OLD.id;
END;;

CREATE TABLE Classifiers 
  ( 
     id        VARCHAR PRIMARY KEY NOT NULL, 
     eventId   VARCHAR NOT NULL, 
     naturalId VARCHAR, 
     name      VARCHAR, 
     info      VARCHAR, 
     typeId    VARCHAR REFERENCES Types(id) DEFERRABLE INITIALLY DEFERRED, 
     ordering  INTEGER DEFAULT 0,
     FOREIGN KEY (eventId) REFERENCES Events(id) ON DELETE CASCADE DEFERRABLE INITIALLY DEFERRED
  );; 
  
CREATE TRIGGER Classifiers_Object_Delete AFTER DELETE ON Classifiers
BEGIN
    DELETE FROM Objects WHERE Objects.id = OLD.id;
END;;

CREATE TABLE Classifications 
  ( 
     eventId         VARCHAR NOT NULL, 
     objectId        VARCHAR NOT NULL REFERENCES Objects(id) ON DELETE CASCADE DEFERRABLE INITIALLY DEFERRED, 
     classifierId    VARCHAR NOT NULL REFERENCES Classifiers(id) DEFERRABLE INITIALLY DEFERRED, 
     ordering        INTEGER DEFAULT 0, 
     inverseOrdering INTEGER DEFAULT 0, 
     PRIMARY KEY (objectId, classifierId),
     FOREIGN KEY (eventId) REFERENCES Events(id) ON DELETE CASCADE DEFERRABLE INITIALLY DEFERRED
  );; 

CREATE TABLE Attachments 
  ( 
     id                 VARCHAR PRIMARY KEY NOT NULL, 
     eventId            VARCHAR NOT NULL, 
     naturalId          VARCHAR, 
     name               VARCHAR, 
     info               VARCHAR, 
     offlineFileName    VARCHAR, 
     visible            BOOLEAN, 
     typeId             VARCHAR REFERENCES Types(id) DEFERRABLE INITIALLY DEFERRED, 
     mimeType           VARCHAR, 
     contentLocation    VARCHAR, 
     isAvailableOffline BOOLEAN DEFAULT 0, 
     ordering           INTEGER DEFAULT 0,
     FOREIGN KEY (eventId) REFERENCES Events(id) ON DELETE CASCADE DEFERRABLE INITIALLY DEFERRED
  );; 

CREATE TRIGGER Attachments_Object_Delete AFTER DELETE ON Attachments
BEGIN
    DELETE FROM Objects WHERE Objects.id = OLD.id;
END;;

CREATE TABLE Attacheds 
  ( 
     eventId      VARCHAR NOT NULL, 
     objectId     VARCHAR NOT NULL REFERENCES Objects(id) ON DELETE CASCADE DEFERRABLE INITIALLY DEFERRED, 
     attachmentId VARCHAR NOT NULL REFERENCES Attachments(id) DEFERRABLE INITIALLY DEFERRED, 
     ordering     INTEGER DEFAULT 0, 
     PRIMARY KEY (objectId, attachmentId),
     FOREIGN KEY (eventId) REFERENCES Events(id) ON DELETE CASCADE DEFERRABLE INITIALLY DEFERRED
  );;

CREATE TABLE Floorplans 
  ( 
     id        VARCHAR PRIMARY KEY NOT NULL, 
     eventId   VARCHAR NOT NULL, 
     naturalId VARCHAR, 
     name      VARCHAR, 
     image     VARCHAR, 
     typeId    VARCHAR REFERENCES Types(id) DEFERRABLE INITIALLY DEFERRED, 
     ordering  INTEGER DEFAULT 0,
     FOREIGN KEY (eventId) REFERENCES Events(id) ON DELETE CASCADE DEFERRABLE INITIALLY DEFERRED
  );; 

CREATE TRIGGER Floorplans_Object_Delete AFTER DELETE ON Floorplans
BEGIN
    DELETE FROM Objects WHERE Objects.id = OLD.id;
END;;

CREATE TABLE Places 
  ( 
     id          VARCHAR PRIMARY KEY NOT NULL, 
     eventId     VARCHAR NOT NULL, 
     naturalId   VARCHAR, 
     name        VARCHAR, 
     info        VARCHAR, 
     positionX   DOUBLE, 
     positionY   DOUBLE, 
     floorplanId VARCHAR REFERENCES Floorplans(id) DEFERRABLE INITIALLY DEFERRED, 
     typeId      VARCHAR REFERENCES Types(id) DEFERRABLE INITIALLY DEFERRED, 
     ordering    INTEGER DEFAULT 0,
     FOREIGN KEY (eventId) REFERENCES Events(id) ON DELETE CASCADE DEFERRABLE INITIALLY DEFERRED
  );; 

CREATE TRIGGER Places_Object_Delete AFTER DELETE ON Places
BEGIN
    DELETE FROM Objects WHERE Objects.id = OLD.id;
END;;

CREATE TABLE Locations 
  ( 
     eventId         VARCHAR NOT NULL, 
     objectId        VARCHAR NOT NULL REFERENCES Objects(id) ON DELETE CASCADE DEFERRABLE INITIALLY DEFERRED, 
     placeId         VARCHAR NOT NULL REFERENCES Places(id) DEFERRABLE INITIALLY DEFERRED, 
     typeId          VARCHAR REFERENCES Types(id) DEFERRABLE INITIALLY DEFERRED, 
     ordering        INTEGER DEFAULT 0, 
     inverseOrdering INTEGER DEFAULT 0, 
     PRIMARY KEY (objectId, placeId),
     FOREIGN KEY (eventId) REFERENCES Events(id) ON DELETE CASCADE DEFERRABLE INITIALLY DEFERRED
  );; 

CREATE TABLE Abstracts 
  ( 
     id        VARCHAR PRIMARY KEY NOT NULL, 
     eventId   VARCHAR NOT NULL, 
     naturalId VARCHAR, 
     name      VARCHAR, 
     subName   VARCHAR, 
     info      VARCHAR, 
     subInfo   VARCHAR, 
     rolesText VARCHAR, 
     typeId    VARCHAR REFERENCES Types(id) DEFERRABLE INITIALLY DEFERRED, 
     ordering  INTEGER DEFAULT 0,
     FOREIGN KEY (eventId) REFERENCES Events(id) ON DELETE CASCADE DEFERRABLE INITIALLY DEFERRED
  );; 

CREATE TRIGGER Abstracts_Object_Delete AFTER DELETE ON Abstracts
BEGIN
    DELETE FROM Objects WHERE Objects.id = OLD.id;
END;;

CREATE TABLE Subjects 
  ( 
     eventId         VARCHAR NOT NULL, 
     timeslotId      VARCHAR NOT NULL REFERENCES Timeslots(id) DEFERRABLE INITIALLY DEFERRED, 
     abstractId      VARCHAR NOT NULL REFERENCES Abstracts(id) DEFERRABLE INITIALLY DEFERRED, 
     typeId          VARCHAR REFERENCES Types(id) DEFERRABLE INITIALLY DEFERRED, 
     ordering        INTEGER DEFAULT 0, 
     inverseOrdering INTEGER DEFAULT 0, 
     PRIMARY KEY (timeslotId, abstractId),
     FOREIGN KEY (eventId) REFERENCES Events(id) ON DELETE CASCADE DEFERRABLE INITIALLY DEFERRED
  );; 

CREATE TABLE Persons 
  ( 
     id               VARCHAR PRIMARY KEY NOT NULL, 
     eventId          VARCHAR NOT NULL, 
     naturalId        VARCHAR, 
     name             VARCHAR, 
     info             VARCHAR, 
     lastName         VARCHAR, 
     affiliationsText VARCHAR, 
     typeId           VARCHAR REFERENCES Types(id) DEFERRABLE INITIALLY DEFERRED, 
     ordering         INTEGER DEFAULT 0,
     FOREIGN KEY (eventId) REFERENCES Events(id) ON DELETE CASCADE DEFERRABLE INITIALLY DEFERRED
  );; 

CREATE TRIGGER Persons_Object_Delete AFTER DELETE ON Persons
BEGIN
    DELETE FROM Objects WHERE Objects.id = OLD.id;
END;;

CREATE TABLE Institutions 
  ( 
     id        VARCHAR PRIMARY KEY NOT NULL, 
     eventId   VARCHAR NOT NULL, 
     naturalId VARCHAR, 
     name      VARCHAR, 
     info      VARCHAR, 
     typeId    VARCHAR REFERENCES Types(id) DEFERRABLE INITIALLY DEFERRED, 
     ordering  INTEGER DEFAULT 0,
     FOREIGN KEY (eventId) REFERENCES Events(id) ON DELETE CASCADE DEFERRABLE INITIALLY DEFERRED
  );; 

CREATE TRIGGER Institutions_Object_Delete AFTER DELETE ON Institutions
BEGIN
    DELETE FROM Objects WHERE Objects.id = OLD.id;
END;;

CREATE TABLE Affiliations 
  ( 
     eventId         VARCHAR NOT NULL, 
     personId        VARCHAR NOT NULL REFERENCES Persons(id) DEFERRABLE INITIALLY DEFERRED, 
     institutionId   VARCHAR NOT NULL REFERENCES Institutions(id) DEFERRABLE INITIALLY DEFERRED, 
     typeId          VARCHAR REFERENCES Types(id) DEFERRABLE INITIALLY DEFERRED, 
     ordering        INTEGER DEFAULT 0, 
     inverseOrdering INTEGER DEFAULT 0, 
     PRIMARY KEY (personId, institutionId),
     FOREIGN KEY (eventId) REFERENCES Events(id) ON DELETE CASCADE DEFERRABLE INITIALLY DEFERRED
  );; 

CREATE TABLE Exhibitors 
  ( 
     id            VARCHAR PRIMARY KEY NOT NULL, 
     eventId       VARCHAR NOT NULL, 
     naturalId     VARCHAR, 
     institutionId VARCHAR NOT NULL REFERENCES Institutions(id) DEFERRABLE INITIALLY DEFERRED, 
     typeId        VARCHAR REFERENCES Types(id) DEFERRABLE INITIALLY DEFERRED, 
     ordering      INTEGER DEFAULT 0,
     FOREIGN KEY (eventId) REFERENCES Events(id) ON DELETE CASCADE DEFERRABLE INITIALLY DEFERRED
  );; 

CREATE TRIGGER Exhibitors_Object_Delete AFTER DELETE ON Exhibitors
BEGIN
    DELETE FROM Objects WHERE Objects.id = OLD.id;
END;;

CREATE TABLE Timeslots 
  ( 
     id        VARCHAR PRIMARY KEY NOT NULL, 
     eventId   VARCHAR NOT NULL, 
     naturalId VARCHAR, 
     name      VARCHAR, 
     subName   VARCHAR, 
     info      VARCHAR, 
     subInfo   VARCHAR, 
     rolesText VARCHAR, 
     start     DATETIME, 
     end       DATETIME, 
     parentId  VARCHAR REFERENCES Timeslots(id) DEFERRABLE INITIALLY DEFERRED, 
     typeId    VARCHAR REFERENCES Types(id) DEFERRABLE INITIALLY DEFERRED, 
     ordering  INTEGER DEFAULT 0,
     FOREIGN KEY (eventId) REFERENCES Events(id) ON DELETE CASCADE DEFERRABLE INITIALLY DEFERRED
  );; 

CREATE TRIGGER Timeslots_Object_Delete AFTER DELETE ON Timeslots
BEGIN
    DELETE FROM Objects WHERE Objects.id = OLD.id;
END;;

CREATE TABLE Roles 
  ( 
     id              VARCHAR PRIMARY KEY NOT NULL, 
     eventId         VARCHAR NOT NULL, 
     naturalId       VARCHAR, 
     programId       VARCHAR NOT NULL REFERENCES Objects(id) DEFERRABLE INITIALLY DEFERRED, 
     actorId         VARCHAR NOT NULL REFERENCES Objects(id) DEFERRABLE INITIALLY DEFERRED, 
     ordering        INTEGER DEFAULT 0, 
     inverseOrdering INTEGER DEFAULT 0, 
     typeId          VARCHAR REFERENCES Types(id) DEFERRABLE INITIALLY DEFERRED,
     FOREIGN KEY (eventId) REFERENCES Events(id) ON DELETE CASCADE DEFERRABLE INITIALLY DEFERRED
  );; 

CREATE TRIGGER Roles_Object_Delete AFTER DELETE ON Roles
BEGIN
    DELETE FROM Objects WHERE Objects.id = OLD.id;
END;;

CREATE TABLE Settings 
  ( 
     id        VARCHAR PRIMARY KEY NOT NULL, 
     eventId   VARCHAR NOT NULL, 
     naturalId VARCHAR, 
     key       VARCHAR, 
     value     VARCHAR,
     FOREIGN KEY (eventId) REFERENCES Events(id) ON DELETE CASCADE DEFERRABLE INITIALLY DEFERRED
  );;

CREATE TRIGGER Settings_Object_Delete AFTER DELETE ON Settings
BEGIN
    DELETE FROM Objects WHERE Objects.id = OLD.id;
END;;

CREATE TABLE Webpages 
  ( 
     id        VARCHAR PRIMARY KEY NOT NULL, 
     eventId   VARCHAR NOT NULL, 
     naturalId VARCHAR, 
     name      VARCHAR, 
     html      VARCHAR, 
     typeId    VARCHAR REFERENCES Types(id) DEFERRABLE INITIALLY DEFERRED, 
     ordering  INTEGER DEFAULT 0,
     FOREIGN KEY (eventId) REFERENCES Events(id) ON DELETE CASCADE DEFERRABLE INITIALLY DEFERRED
  );;
  
CREATE TRIGGER Webpages_Object_Delete AFTER DELETE ON Webpages
BEGIN
    DELETE FROM Objects WHERE Objects.id = OLD.id;
END;;

CREATE TABLE Extras 
  ( 
     id       VARCHAR PRIMARY KEY NOT NULL, 
     eventId  VARCHAR NOT NULL, 
     objectId VARCHAR REFERENCES Objects(id) ON DELETE CASCADE DEFERRABLE INITIALLY DEFERRED, 
     key      VARCHAR, 
     value    VARCHAR,
     FOREIGN KEY (eventId) REFERENCES Events(id) ON DELETE CASCADE DEFERRABLE INITIALLY DEFERRED
  );;

CREATE TABLE FavoriteObjects 
  ( 
     objectId             VARCHAR NOT NULL PRIMARY KEY REFERENCES Objects(id) ON DELETE CASCADE DEFERRABLE INITIALLY DEFERRED, 
     eventId              VARCHAR NOT NULL, 
     isFavorite           INTEGER DEFAULT 0, 
     recommendation       REAL DEFAULT 0, 
     ignoreRecommendation INTEGER DEFAULT 0, 
     x1                   INTEGER DEFAULT 0, 
     lastUpdate           DATETIME DEFAULT CURRENT_TIMESTAMP, 
     FOREIGN KEY (eventId) REFERENCES Events(id) ON DELETE CASCADE DEFERRABLE INITIALLY DEFERRED
  );;

CREATE TABLE Tags 
  ( 
     id         VARCHAR PRIMARY KEY NOT NULL, 
     name       VARCHAR(50), 
     lastUpdate DATETIME DEFAULT CURRENT_TIMESTAMP, 
     deleted    INTEGER DEFAULT 0 
  );;

CREATE TABLE Taggeds 
  ( 
     objectId   VARCHAR NOT NULL PRIMARY KEY REFERENCES Objects(id) ON DELETE CASCADE DEFERRABLE INITIALLY DEFERRED, 
     eventId    VARCHAR NOT NULL, 
     tagId      VARCHAR NOT NULL REFERENCES Tags(id) ON DELETE CASCADE DEFERRABLE INITIALLY DEFERRED, 
     lastUpdate DATETIME DEFAULT CURRENT_TIMESTAMP, 
     deleted    INTEGER DEFAULT 0,
     FOREIGN KEY (eventId) REFERENCES Events(id) ON DELETE CASCADE DEFERRABLE INITIALLY DEFERRED
  );;

CREATE TABLE Notes 
  ( 
     id         VARCHAR PRIMARY KEY NOT NULL, 
     eventId    VARCHAR NOT NULL, 
     text       VARCHAR(300), 
     objectId   VARCHAR REFERENCES Objects(id) ON DELETE CASCADE DEFERRABLE INITIALLY DEFERRED, 
     lastUpdate DATETIME DEFAULT CURRENT_TIMESTAMP, 
     deleted    INTEGER DEFAULT 0,
     FOREIGN KEY (eventId) REFERENCES Events(id) ON DELETE CASCADE DEFERRABLE INITIALLY DEFERRED
  );;

CREATE INDEX Objects_objectClass 
  ON Objects (objectClass COLLATE BINARY ASC);;

CREATE INDEX Types_naturalId 
  ON Types (naturalId COLLATE BINARY ASC);;

CREATE INDEX Classifiers_naturalId 
  ON Classifiers (naturalId COLLATE BINARY ASC);;

CREATE INDEX Classifiers_typeId 
  ON Classifiers (typeId COLLATE BINARY ASC);;

CREATE INDEX Classifications_objectId 
  ON Classifications (objectId COLLATE BINARY ASC);;

CREATE INDEX Classifications_classifierId 
  ON Classifications (classifierId COLLATE BINARY ASC);;

CREATE INDEX Attachments_naturalId 
  ON Attachments (naturalId COLLATE BINARY ASC);;

CREATE INDEX Attacheds_objectId 
  ON Attacheds (objectId COLLATE BINARY ASC);;

CREATE INDEX Places_floorplanId 
  ON Places (floorplanId COLLATE BINARY ASC);;

CREATE INDEX Locations_objectId 
  ON Locations (objectId COLLATE BINARY ASC);;

CREATE INDEX Subjects_timeslotId 
  ON Subjects (timeslotId COLLATE BINARY ASC);;

CREATE INDEX Affiliations_personId 
  ON Affiliations (personId COLLATE BINARY ASC);;

CREATE INDEX Exhibitors_institutionId 
  ON Exhibitors (institutionId COLLATE BINARY ASC);;

CREATE INDEX Timeslots_parentId 
  ON Timeslots (parentId COLLATE BINARY ASC);;

CREATE INDEX Timeslots_typeId 
  ON Timeslots (typeId COLLATE BINARY ASC);;

CREATE INDEX Roles_programId 
  ON Roles (programId COLLATE BINARY ASC);;

CREATE INDEX Roles_actorId 
  ON Roles (actorId COLLATE BINARY ASC);;

CREATE INDEX Roles_typeId 
  ON Roles (typeId COLLATE BINARY ASC);;

CREATE INDEX Webpages_naturalId 
  ON Webpages (naturalId COLLATE BINARY ASC);;

CREATE INDEX FavoriteObjects_objectId 
  ON FavoriteObjects (objectId COLLATE BINARY ASC);;

CREATE INDEX FavoriteObjects_isFavorite 
  ON FavoriteObjects (isFavorite COLLATE BINARY ASC);;

CREATE INDEX Taggeds_objectId 
  ON Taggeds (objectId COLLATE BINARY ASC);;

CREATE INDEX Notes_objectId 
  ON Notes (objectId COLLATE BINARY ASC);;